/*
 * spi.c
 *
 *  Created on: Nov 6, 2022
 *      Author: hussamaldean
 */


#include "spi.h"
#include "stm32f4xx.h"


void st7920_spi_pins_init()
{
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN; //enable clock for GPIOA
	GPIOA->MODER|=GPIO_MODER_MODE0_0;
	GPIOA->MODER&=~GPIO_MODER_MODE0_1;

	GPIOA->MODER|=GPIO_MODER_MODE1_0;
	GPIOA->MODER&=~GPIO_MODER_MODE1_1;

	GPIOA->MODER|=GPIO_MODER_MODE4_0;
	GPIOA->MODER&=~GPIO_MODER_MODE4_1;

	GPIOA->MODER |=(GPIO_MODER_MODE5_1|GPIO_MODER_MODE6_1|GPIO_MODER_MODE7_1);
	GPIOA->MODER &=~(GPIO_MODER_MODE5_0|GPIO_MODER_MODE6_0|GPIO_MODER_MODE7_0);

	#define SPI1_AF 0x05

	GPIOA->AFR[0]|=(SPI1_AF<<GPIO_AFRL_AFSEL5_Pos)|(SPI1_AF<<GPIO_AFRL_AFSEL6_Pos)|(SPI1_AF<<GPIO_AFRL_AFSEL7_Pos);
}

void st7920_spi_config()
{
	/*Enable clock access to SPI1 module*/
	RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;

	/*Set MSB first*/
	SPI1->CR1 &=~ SPI_CR1_LSBFIRST;

	/*Set mode to MASTER*/
	SPI1->CR1 |= SPI_CR1_MSTR;

	/*Select software slave management by
	 * setting SSM=1 and SSI=1*/
	SPI1->CR1 |= SPI_CR1_SSM;
	SPI1->CR1 |= SPI_CR1_SSI;

	/*Set SPI mode to be MODE1 (CPHA1 CPOL0)*/
	SPI1->CR1|=SPI_CR1_CPHA;

	/*Enable SPI module*/
	SPI1->CR1 |= SPI_CR1_SPE;
}

void st7920_spi_transmit(uint8_t *data,uint32_t size)
{
	uint32_t i=0;

	while(i<size)
	{
		/*Wait until TXE is set*/
		while(!(SPI1->SR & (SPI_SR_TXE))){}

		/*Write the data to the data register*/
		SPI1->DR = data[i];
		i++;
	}
	/*Wait until TXE is set*/
	while(!(SPI1->SR & (SPI_SR_TXE))){}

	/*Wait for BUSY flag to reset*/
	while((SPI1->SR & (SPI_SR_BSY))){}

	/*Clear OVR flag*/
	(void)SPI1->DR;
	(void)SPI1->SR;
}


